# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from thx.api.mixins import ThuxModelMixin
from thx.role.core.views import ThuxRoleViewSetMixin

from ... import models
from . import serializers, filters, permissions

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# ADDRESS SCOPE
class AddressScopeViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    AddressScopeViewSet
    -----------

    :action list: Return the address scope list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given address scope.
        `Profiles`: `superuser`, `staff`

    :action light: Return the address scope list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new address scope instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a address scope instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a address scope instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a address scope instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.AddressScope.objects.all()
    serializer_class = serializers.AddressScopeSerializer
    filter_backends = (filters.AddressScopeFilterBackend,)
    permission_classes = (permissions.AddressScopePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a address scope instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a address scope instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ADDRESS
class AddressViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    AddressViewSet
    -----------

    :action list: Return the address list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given address.
        `Profiles`: `superuser`, `staff`

    :action light: Return the address list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new address instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a address instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a address instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a address instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Address.objects.all()
    serializer_class = serializers.AddressSerializer
    filter_backends = (filters.AddressFilterBackend,)
    permission_classes = (permissions.AddressPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a address instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a address instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ADDRESS COORDINATE
class AddressCoordinateViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    AddressCoordinateViewSet
    -----------

    :action list: Return the address coordinate list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given address coordinate.
        `Profiles`: `superuser`, `staff`

    :action light: Return the address coordinate list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new address coordinate instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a address coordinate instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a address coordinate instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a address coordinate instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.AddressCoordinate.objects.all()
    serializer_class = serializers.AddressCoordinateSerializer
    filter_backends = (filters.AddressCoordinateFilterBackend,)
    permission_classes = (permissions.AddressCoordinatePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a address coordinate instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a address coordinate instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
